"""Provide file handling."""

import copy
import re
from io import open
from os import access, W_OK, close, path
from tempfile import mkstemp
from .config import TlpConfig, ConfType
from . import settings
from . import settingshelper
from .filehelper import get_yaml_schema_object_from_file, extract_default_tlp_configs, TlpDefaults
from .uihelper import get_graphical_sudo


def get_yaml_schema_object(objectname) -> dict:
    """Get Json schema for installed TLP version."""
    tlpprovidedschema = '/usr/share/tlp-pm/configschema.yaml'
    if path.exists(tlpprovidedschema):
        return get_yaml_schema_object_from_file(objectname, tlpprovidedschema)
    majorminor = settings.tlpbaseversion
    return get_yaml_schema_object_from_file(objectname, f"{settings.workdir}/configschema/{majorminor}.yaml")


def get_tlp_config_defaults(tlpversion: str):
    """Fetch TLP default configs."""
    tlpconfig_defaults = extract_default_tlp_configs(f"{settings.workdir}/defaults/tlp-{tlpversion}.conf")

    # update default values with intrinsic ones
    intrinsic_defaults_path = f"{settings.FOLDER_PREFIX}/usr/share/tlp/defaults.conf"
    tlpconfig_defaults.update(extract_default_tlp_configs(intrinsic_defaults_path))

    return tlpconfig_defaults


def init_tlp_file_config() -> None:
    """Load current TLP config settings."""
    settings.tlpconfig = {}
    tlpversion = settings.tlpbaseversion
    settings.tlpconfig_defaults = get_tlp_config_defaults(tlpversion)

    # get current settings from tlp itself
    tlpstat = settingshelper.exec_command(["tlp-stat", "-c"])
    tlpsettinglines = tlpstat.split('\n')

    extract_tlp_settings(tlpsettinglines)

    # Add default tlp configs not set in current settings
    for key, default in settings.tlpconfig_defaults.items():     # type: TlpDefaults
        configname = default.get_name()
        if configname not in settings.tlpconfig.keys():
            enabled = default.is_enabled()
            value = default.get_value()
            settings.tlpconfig[configname] = TlpConfig(enabled, configname, value, ConfType.DEFAULT, "")

    # finally store copy for comparing changes
    settings.tlpconfig_original = copy.deepcopy(settings.tlpconfig)


def extract_tlp_settings(lines: list) -> None:
    """Extract TLP config locations and values."""
    propertypattern = re.compile(r'^.+?\.conf\sL\d+\:\s[A-Z_\d]+=')

    for line in lines:
        if propertypattern.match(line):
            cleanline = line.lstrip().rstrip()
            settingsparts = cleanline.split(' ', maxsplit=2)
            configfile = settingsparts[0]       # type: str
            if configfile == 'defaults.conf':
                conftype = ConfType.DEFAULT
            elif configfile.startswith('/etc/tlp.d/'):
                conftype = ConfType.DROPIN
            elif configfile == "/etc/tlp.conf":
                conftype = ConfType.USER
            else:
                print(f'Config type not found for file: {configfile}')
                conftype = ConfType.ERR

            configproperty = settingsparts[2].split('=', maxsplit=1)
            configname = configproperty[0]
            configvalue = configproperty[1]

            if configvalue.startswith('\"') and configvalue.endswith('\"'):
                configvalue = configvalue.lstrip('\"').rstrip('\"')

            enabled = configvalue != ""

            settings.tlpconfig[configname] = TlpConfig(enabled, configname, configvalue, conftype, configfile)


def get_changed_properties() -> dict:
    """Evaluate changed settings from UI."""
    changedproperties = {}

    changed = settings.tlpconfig
    original = settings.tlpconfig_original

    for configid in changed:
        config = changed[configid]              # type: TlpConfig
        config_original = original[configid]    # type: TlpConfig

        statechange = config.is_enabled() != config_original.is_enabled()
        configchange = config.get_value() != config_original.get_value()

        if statechange or configchange:
            configname = config.get_name()
            value = config.get_value()

            if not config.is_enabled() and settings.tlpconfig_defaults[configname].is_enabled():
                enabled = ""
            else:
                enabled = "" if config.is_enabled() else "#"

            if settings.tlpconfig_defaults[configname].is_quoted():
                value = f"\"{value}\""

            changedproperties[configname] = f"{enabled}{configname}={value}"

    return changedproperties


def create_tmp_tlp_config_file(changedproperties: dict) -> str:
    """Create tmp file to prepare writing new config."""
    propertypattern = re.compile(r'^#?[A-Z_\d]+=')
    filehandler, tmpfilename = mkstemp(dir=settings.TMP_FOLDER)
    newfile = open(tmpfilename, mode='w', encoding='utf-8')

    oldfile = open(settings.tlpconfigfile, encoding='utf-8')
    lines = oldfile.readlines()
    oldfile.close()

    for line in lines:
        if propertypattern.match(line):
            cleanline = line.lstrip().lstrip('#')

            configproperty = cleanline.split('=', maxsplit=1)
            configname = configproperty[0]

            if configname in changedproperties.keys():
                if cleanline.startswith(configname + "="):
                    newfile.write(changedproperties[configname] + '\n')
                    continue

        newfile.write(line)

    newfile.close()
    close(filehandler)
    oldfile.close()

    return tmpfilename


def write_tlp_config(tmpconfigfile: str):
    """Write changes to config file."""
    sedtlpconfigfile = "w" + settings.tlpbaseconfigfile
    sedcommand = ["sed", "-n", sedtlpconfigfile, tmpconfigfile]

    # check permission and apply sudo if needed
    if not access(settings.tlpconfigfile, W_OK):
        sudo_cmd = get_graphical_sudo()
        if sudo_cmd is None:
            return
        sedcommand.insert(0, sudo_cmd)
    settingshelper.exec_command(sedcommand)



"""PCI UI widget."""

import re
from gi.repository import Gtk

from collections import OrderedDict
from ..uihelper import get_theme_image
from .. import settings
from .. import settingshelper


def create_list(configname: str, window: Gtk.Window) -> Gtk.Box:
    """Create pci list button."""
    box = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
    label = Gtk.Label(settings.tlpconfig[configname].get_value().replace(" ", "\n"))

    button = Gtk.Button(label=' Edit', image=get_theme_image('edit-symbolic', Gtk.IconSize.BUTTON))
    button.connect('clicked', edit_list, configname, label, window)
    button.set_always_show_image(True)

    box.pack_start(label, False, False, 0)
    box.pack_start(button, False, False, 12)
    return box


def edit_list(self, configname: str, usblistlabel: Gtk.Label, window: Gtk.Window):
    """Create pci list view."""
    tlpobject = settings.tlpconfig[configname]
    pcilistpattern = re.compile(r'^([a-f\d]{2}:[a-f\d]{2}\.[a-f\d])(.+?)$')
    currentitems = tlpobject.get_value().split(' ')

    tlppcilist = settingshelper.exec_command(["lspci"])

    pciitems = OrderedDict()
    for line in tlppcilist.splitlines():
        matcher = pcilistpattern.match(line)
        pciid = matcher.group(1)
        description = matcher.group(2).lstrip()

        pciitems[pciid] = [description, (pciid in currentitems)]

    grid = Gtk.Grid()
    grid.set_row_homogeneous(True)
    grid.set_column_spacing(12)

    grid.attach(Gtk.Label(''), 0, 0, 1, 1)
    grid.attach(Gtk.Label(label='ID', halign=Gtk.Align.START), 1, 0, 1, 1)
    grid.attach(Gtk.Label(label='Description', halign=Gtk.Align.START), 2, 0, 1, 1)
    grid.attach(Gtk.Label(''), 3, 0, 1, 1)

    rowindex = 2
    allitems = list()
    selecteditems = list()
    for key, value in pciitems.items():
        allitems.append(key)
        toggle = Gtk.ToggleButton(key)
        toggle.connect('toggled', on_button_toggled, key, selecteditems)

        if value[1]:
            toggle.set_active(True)

        label = Gtk.Label(value[0])
        label.set_halign(Gtk.Align.START)

        grid.attach(toggle, 1, rowindex, 1, 1)
        grid.attach(label, 2, rowindex, 1, 1)

        rowindex += 1

    dialog = Gtk.Dialog('PCI(e) devices', window, 0, (
        Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
        Gtk.STOCK_OK, Gtk.ResponseType.OK
    ))

    contentarea = dialog.get_content_area()
    contentarea.set_spacing(6)
    contentarea.add(grid)
    dialog.show_all()

    response = dialog.run()
    if response == Gtk.ResponseType.OK:
        configvalue = ' '.join(str(item) for item in selecteditems)
        tlpobject.set_value(configvalue)
        usblistlabel.set_text(configvalue.replace(" ", "\n"))

    dialog.destroy()


def on_button_toggled(self: Gtk.ToggleButton, key: str, selecteditems: list):
    """Process state change."""
    if self.get_active():
        selecteditems.append(key)
    else:
        selecteditems.remove(key)



"""Create TLP config UI."""

import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk

from collections import OrderedDict
from . import settings
from . import language
from .ui_config_objects import gtkswitch, gtkentry, gtkselection, gtkmultiselection, gtkcheckbutton, gtkspinbutton, \
    gtktoggle, gtkusblist, gtkpcilist, gtkdisklist, gtkdisklistview
from .file import ConfType, TlpConfig, get_yaml_schema_object
from .uihelper import get_theme_image, StateImage, EXPECTED_ITEM_MISSING_TEXT


def store_category_num(self, cat, cat_num: int):
    """Store selected config category."""
    settings.userconfig.activecategory = cat_num
    # settings.userconfig.activeposition = self.get_children()[cat_num].get_vadjustment().get_value()
    settings.active_scroll = self.get_children()[cat_num]


def store_scroll_position(self, event):
    """Store current scrolled position."""
    settings.userconfig.activeposition = self.get_vadjustment().get_value()


def create_config_box(window) -> Gtk.Box:
    """Create TLP config box."""
    notebook = Gtk.Notebook()
    notebook.set_name('configNotebook')
    notebook.set_tab_pos(Gtk.PositionType.LEFT)

    tlp_categories = get_tlp_categories(window)
    for name, category_data in tlp_categories.items():
        viewport = Gtk.Viewport()
        viewport.set_name(f'categoryViewport_{name}')
        viewport.add(category_data[1])

        scroll = Gtk.ScrolledWindow()
        scroll.add(viewport)
        scroll.connect("scroll-event", store_scroll_position)

        category_image = get_theme_image(f'tlpui-{name.replace(" ", "-")}-symbolic', Gtk.IconSize.MENU)

        labelbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=0)
        labelbox.pack_start(category_image, False, False, 0)
        labelbox.pack_start(category_data[0], True, True, 0)
        labelbox.show_all()

        notebook.append_page(scroll, labelbox)

    notebook.connect('switch-page', store_category_num)

    containerbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=0)
    containerbox.pack_start(notebook, True, True, 0)

    activecategory = settings.userconfig.activecategory
    notebook.show_all()
    notebook.set_current_page(activecategory)

    return containerbox


def create_config_widget(configname: str, objecttype: str, objectvalues: str, window: Gtk.Window) -> Gtk.Widget:
    """Create config widget."""
    configwidget = Gtk.Widget

    if objecttype == 'entry':
        configwidget = gtkentry.create_entry(configname)
    elif objecttype == 'usblist':
        configwidget = gtkusblist.create_list(configname, window)
    elif objecttype == 'pcilist':
        configwidget = gtkpcilist.create_list(configname, window)
    elif objecttype == 'disklist':
        configwidget = gtkdisklist.create_list(configname, window)
    elif objecttype == 'disklistview':
        configwidget = gtkdisklistview.create_view(configname)
    elif objecttype == 'bselect':
        configwidget = gtkswitch.create_state_switch(configname, objectvalues)
    elif objecttype == 'select':
        configwidget = gtkselection.create_selection_box(configname, objectvalues)
    elif objecttype == 'multiselect':
        configwidget = gtkmultiselection.create_multi_selection_box(configname, objectvalues)
    elif objecttype == 'check':
        configwidget = gtkcheckbutton.create_checkbutton_box(configname, objectvalues)
    elif objecttype == 'numeric':
        configwidget = gtkspinbutton.create_numeric_spinbutton(configname, objectvalues)

    return configwidget


def init_state_image(configname: str):
    """Create and store state image."""
    image = Gtk.Image()
    defaultvalue = settings.tlpconfig_defaults[configname].get_value()
    defaultstate = settings.tlpconfig_defaults[configname].is_enabled()
    settings.tlpconfig[configname].add_state_image(StateImage(defaultvalue, defaultstate, image))
    return image


def get_type_image(configname: str) -> Gtk.Image:
    """Create config location image."""
    tlpconfig = settings.tlpconfig[configname]      # type: TlpConfig
    conftype = tlpconfig.get_conf_type()
    conftypeimage = Gtk.Image()

    if conftype == ConfType.DROPIN:
        conftypeimage = Gtk.Image.new_from_file(f'{settings.icondir}dropin.svg')
        conftypeimage.set_tooltip_text(tlpconfig.get_conf_path())
    elif conftype == ConfType.USER:
        conftypeimage = Gtk.Image.new_from_file(f'{settings.icondir}user.svg')
        conftypeimage.set_tooltip_text(tlpconfig.get_conf_path())

    return conftypeimage


def create_item_box(configobjects: list, doc: str, grouptitle: str, window) -> Gtk.Box:
    """Create box with config item widgets."""
    configuibox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=0)

    if len(configobjects) > 1:
        grouplabel = Gtk.Label()
        grouplabel.set_markup(f' <b>{grouptitle.replace("_", " ")}</b> ')
        grouplabel.set_use_markup(True)
        grouplabel.set_margin_bottom(12)
        grouplabel.set_halign(Gtk.Align.START)
        grouplabel.set_valign(Gtk.Align.START)

        configuibox.pack_start(grouplabel, False, False, 0)

    for configobject in configobjects:      # type: ConfigObject
        configname = configobject.name
        stateimage = init_state_image(configname)
        tlpuiobject = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=18)
        tlpuiobject.set_margin_start(18)
        tlpuiobject.set_margin_end(18)

        if configname not in settings.tlpconfig.keys():
            missingcheckbox = Gtk.CheckButton()
            missingcheckbox.set_child_visible(False)
            missingstatetogglebox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=0)
            missingstatetogglebox.pack_start(missingcheckbox, False, False, 0)
            missingstatetogglebox.set_halign(Gtk.Align.CENTER)
            missingstatetogglebox.set_valign(Gtk.Align.CENTER)

            missingconfiglabel = Gtk.Label(xalign=0)
            missingconfiglabel.set_name('missingConfigLabel')
            missingconfiglabel.set_markup(f' <b>{configname}</b> - <i>{EXPECTED_ITEM_MISSING_TEXT}</i> ')
            missingconfiglabel.set_use_markup(True)

            tlpuiobject.pack_start(missingstatetogglebox, False, False, 0)
            tlpuiobject.pack_start(missingconfiglabel, False, False, 0)
            configuibox.pack_start(tlpuiobject, True, True, 0)
            continue

        statetogglebox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=0)
        tlpconfigbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=0)

        # specific config gtk object
        configwidget = create_config_widget(configname, configobject.datatype, configobject.values, window)
        configwidget.set_margin_top(6)
        configwidget.set_margin_bottom(6)
        configwidget.set_margin_left(6)

        # on/off state
        toggle = gtktoggle.create_toggle_button(configname, configwidget, window)
        statetogglebox.pack_start(toggle, False, False, 0)
        statetogglebox.set_halign(Gtk.Align.CENTER)
        statetogglebox.set_valign(Gtk.Align.CENTER)

        # object label
        configlabel = Gtk.Label(xalign=0)
        configlabel.set_markup(f' <b>{configname}</b> ')
        configlabel.set_use_markup(True)
        configlabel.set_size_request(300, 0)

        # combine boxes
        tlpconfigbox.pack_start(configlabel, False, False, 0)
        tlpconfigbox.pack_start(configwidget, True, True, 0)

        if configname.startswith('CPU_SCALING_MIN_FREQ') or configname.startswith('CPU_SCALING_MAX_FREQ'):
            khzlabel = Gtk.Label()
            khzlabel.set_markup('<small>kHz</small>')
            tlpconfigbox.pack_start(khzlabel, False, False, 12)

        if configname.endswith('_BAT'):
            tlpconfigbox.pack_start(Gtk.Image.new_from_file(f'{settings.icondir}OnBAT.svg'), False, False, 12)
        elif configname.endswith('_AC'):
            tlpconfigbox.pack_start(Gtk.Image.new_from_file(f'{settings.icondir}OnAC.svg'), False, False, 12)

        tlpuiobject.pack_start(statetogglebox, False, False, 0)
        tlpuiobject.pack_start(tlpconfigbox, False, False, 0)
        tlpuiobject.pack_start(stateimage, False, False, 0)
        tlpuiobject.pack_end(get_type_image(configname), False, False, 0)

        configuibox.pack_start(tlpuiobject, True, True, 0)

    # object description
    configdescriptionlabel = Gtk.Label()
    configdescriptionlabel.set_markup(doc)
    configdescriptionlabel.set_line_wrap(True)
    configdescriptionlabel.set_margin_top(6)
    configdescriptionlabel.set_margin_bottom(12)
    configdescriptionlabel.set_margin_start(48)
    configdescriptionlabel.set_halign(Gtk.Align.START)
    configdescriptionlabel.set_valign(Gtk.Align.START)

    # add description and horizontal separator
    configuibox.pack_start(configdescriptionlabel, True, True, 0)
    configuibox.pack_start(Gtk.HSeparator(), True, True, 0)

    return configuibox


def get_tlp_categories(window) -> OrderedDict:
    """Get categories from TLP schema."""
    propertyobjects = OrderedDict()

    categories = get_yaml_schema_object('categories')
    for category in categories:
        categoryname = category['name']

        # Create category label
        categorylabel = Gtk.Label(language.CDT_(f"{categoryname}__CATEGORY_TITLE"))
        categorylabel.set_alignment(1, 0.5)
        categorylabel.set_margin_top(6)
        categorylabel.set_margin_bottom(6)
        categorylabel.set_margin_left(6)
        categorylabel.set_margin_right(6)

        # Create category box
        categorybox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=12)

        configs = category['configs']
        for config in configs:
            grouptitle = ""
            configobjects = []

            if 'group' in config:
                grouptitle = config['group']
                transdescription = f"{grouptitle}__GROUP_DESCRIPTION"
                groupitems = config['ids']
                for groupitem in groupitems:
                    configobjects.append(ConfigObject(groupitem['id'], groupitem['type'], groupitem['values']))
            else:
                itemid = config['id']
                transdescription = f"{itemid}__ID_DESCRIPTION"
                configobjects.append(ConfigObject(itemid, config['type'], config['values']))

            configbox = create_item_box(configobjects, language.CDT_(transdescription), grouptitle, window)
            configbox.set_margin_start(12)
            configbox.set_margin_end(12)
            configbox.set_margin_top(12)
            categorybox.pack_start(configbox, False, False, 0)

        propertyobjects[categoryname] = [categorylabel, categorybox]

    return propertyobjects


class ConfigObject:
    """Config object helper class."""

    def __init__(self, name: str, datatype: str, values: str):
        """Init config object helper class parameters."""
        self.name = name
        self.datatype = datatype
        self.values = values



"""Provide CSS support for UI."""

from gi.repository import Gtk
from . import settings


def get_css_provider() -> Gtk.CssProvider:
    """Create css provider from stylesheet file and return it."""
    with open(f'{settings.workdir}/styles.css', encoding='utf-8') as cssfile:
        cssdata = cssfile.read()

    cssprovider = Gtk.CssProvider()
    cssprovider.load_from_data(cssdata)

    return cssprovider



"""Provide main window parts for TLPUI."""

import importlib
import importlib.metadata
import difflib
import toml

from pathlib import Path
from gi.repository import Gtk, Gdk, GdkPixbuf
from . import settings
from . import language
from .configui import create_config_box
from .file import init_tlp_file_config, create_tmp_tlp_config_file, write_tlp_config, get_changed_properties
from .statui import create_stat_box
from .uihelper import get_flag_image, get_theme_image


def get_app_version() -> str:
    """Get TLP-UI version."""
    try:
        return importlib.metadata.version("tlpui")
    except importlib.metadata.PackageNotFoundError:
        try:
            pyproject = toml.load(Path(__file__).parent.parent / "pyproject.toml")
            return pyproject["tool"]["poetry"]["version"]
        except FileNotFoundError:
            return ""


def reset_scroll_position() -> None:
    """Reset the scroll position."""
    settings.active_scroll.get_vadjustment().set_value(settings.userconfig.activeposition)


def store_window_size(self) -> None:
    """Store current window size in settings."""
    settings.userconfig.windowxsize = self.get_size()[0]
    settings.userconfig.windowysize = self.get_size()[1]


def window_key_events(self, event) -> None:
    """Add window key events like crtl+(q|w|s)."""
    if event.state & Gdk.ModifierType.CONTROL_MASK:
        # close window with ctrl+q or ctrl+w
        if event.keyval in (113, 119):
            quit_tlp_config(None, self)
        # save config with ctrl+s
        if event.keyval == 115:
            save_tlp_config(None, self)


def close_main_window(self, _) -> bool:
    """Close main window."""
    quit_tlp_config(None, self)

    # When delete-event is cancelled we have to return True
    # Otherwise application window will disappear
    return True


def load_tlp_config(_, window: Gtk.Window, reloadtlpconfig: bool) -> None:
    """Load TLP configuration to UI."""
    if reloadtlpconfig:
        init_tlp_file_config()

    new_mainbox = create_main_box(window)
    children = window.get_children()
    for child in children:
        window.remove(child)
    window.add(new_mainbox)
    window.show_all()
    while Gtk.events_pending():
        Gtk.main_iteration()
    reset_scroll_position()


def save_tlp_config(self, window) -> None:
    """Persist TLP configuration changes."""
    changedproperties = get_changed_properties()
    if len(changedproperties) == 0:
        return

    tmpfilename = create_tmp_tlp_config_file(changedproperties)

    saveresponse = changed_items_dialog(
        window,
        tmpfilename,
        language.MT_('Review settings'),
        language.MT_('Save these changes?'))

    if saveresponse == Gtk.ResponseType.OK:
        write_tlp_config(tmpfilename)

        # reload config after file save
        load_tlp_config(self, window, True)


def quit_tlp_config(_, window) -> None:
    """Quit TLPUI and prompt for unsaved changes."""
    settings.userconfig.write_user_config()

    changedproperties = get_changed_properties()
    if len(changedproperties) == 0:
        Gtk.main_quit()
        return

    tmpfilename = create_tmp_tlp_config_file(changedproperties)

    quitresponse = changed_items_dialog(
        window,
        tmpfilename,
        language.MT_('Unsaved settings'),
        language.MT_('Do you really want to quit? No changes will be saved'))

    if quitresponse == Gtk.ResponseType.OK:
        Gtk.main_quit()


def changed_items_dialog(window, tmpfilename: str, dialogtitle: str, message: str) -> Gtk.ResponseType:
    """Dialog to show changed TLP configuration items."""
    dialog = Gtk.Dialog(dialogtitle, window, 0, (
        Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
        Gtk.STOCK_OK, Gtk.ResponseType.OK
    ))
    dialog.set_default_size(400, 300)

    scrolledwindow = Gtk.ScrolledWindow()
    scrolledwindow.set_hexpand(True)
    scrolledwindow.set_vexpand(True)

    with open(settings.tlpconfigfile, encoding='utf-8') as fromfile:
        fromfilecontent = fromfile.readlines()
    with open(tmpfilename, encoding='utf-8') as tofile:
        tofilecontent = tofile.readlines()
    diff = settings.tlpbaseconfigfile + '\n\n'
    for line in difflib.unified_diff(fromfilecontent, tofilecontent, n=0, lineterm=''):
        if line.startswith('---') or line.startswith('+++'):
            continue
        postfix = '' if line.startswith('-') else '\n'
        diff += line + postfix

    textbuffer = Gtk.TextBuffer()
    textbuffer.set_text(diff)

    textview = Gtk.TextView()
    textview.set_buffer(textbuffer)
    textview.set_editable(False)

    scrolledwindow.add(textview)
    scrolledwindow.set_border_width(12)

    box = dialog.get_content_area()
    box.pack_start(scrolledwindow, True, True, 0)
    box.pack_start(Gtk.Label(f'\n{message}\n'), False, False, 0)

    dialog.show_all()
    response = dialog.run()
    dialog.destroy()

    return response


def create_menu_box(window) -> Gtk.Box:
    """Create application menu from XML structure."""
    xmlmenustructure = """
    <ui>
        <menubar name='menubar'>
            <menu action='FileMenu'>
                <menuitem action='save' />
                <menuitem action='quit' />
            </menu>
            <menu name="language_menu" action='LanguageMenu'>
                <menuitem name="en_EN" action='en_EN' />
                <menuitem name="de_DE" action='de_DE' />
                <menuitem name="es_ES" action='es_ES' />
                <menuitem name="fr_FR" action='fr_FR' />
                <menuitem name="pt_BR" action='pt_BR' />
                <menuitem name="ru_RU" action='ru_RU' />
                <menuitem name="tr_TR" action='tr_TR' />
                <menuitem name="id_ID" action='id_ID' />
                <menu name="zh_CN" action='zhSubMenu'>
                    <menuitem name="zh_CN" action='zh_CN' />
                    <menuitem name="zh_TW" action='zh_TW' />
                </menu>
            </menu>
            <menu name="help_menu" action='HelpMenu'>
                <menuitem name="about_dialog" action='AboutDialog' />
            </menu>
        </menubar>
    </ui>
    """

    uimanager = Gtk.UIManager()
    uimanager.add_ui_from_string(xmlmenustructure)

    actiongroup = Gtk.ActionGroup("actions")
    add_menu_actions(window, actiongroup)
    uimanager.insert_action_group(actiongroup)

    menubar = uimanager.get_widget("/menubar")

    repack_language_menuitem(uimanager.get_widget("/menubar/language_menu/en_EN"))
    repack_language_menuitem(uimanager.get_widget("/menubar/language_menu/de_DE"))
    repack_language_menuitem(uimanager.get_widget("/menubar/language_menu/es_ES"))
    repack_language_menuitem(uimanager.get_widget("/menubar/language_menu/fr_FR"))
    repack_language_menuitem(uimanager.get_widget("/menubar/language_menu/pt_BR"))
    repack_language_menuitem(uimanager.get_widget("/menubar/language_menu/ru_RU"))
    repack_language_menuitem(uimanager.get_widget("/menubar/language_menu/tr_TR"))
    repack_language_menuitem(uimanager.get_widget("/menubar/language_menu/id_ID"))
    repack_language_menuitem(uimanager.get_widget("/menubar/language_menu/zh_CN"))

    menubox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL)
    menubox.pack_start(menubar, False, False, 0)

    return menubox


def repack_language_menuitem(menuitem: Gtk.MenuItem):
    """Repack language menu items for better visibility."""
    menuitemname = menuitem.get_name()
    langimage = get_flag_image(menuitemname)
    langlabel = Gtk.Label(menuitemname.split("_")[0])
    langbox = Gtk.Box()
    langbox.pack_start(langimage, False, False, 12)
    langbox.pack_start(langlabel, False, False, 0)
    [menuitem.remove(child) for child in menuitem.get_children()]
    menuitem.add(langbox)


def create_settings_box(window) -> Gtk.Box:
    """Buttons for direct access in UI."""
    fileentrylabel = Gtk.Label(f"TLP {settings.tlpversion} - {settings.tlpbaseconfigfile}")
    fileentrylabel.set_alignment(0, 0.5)
    reloadbutton = Gtk.Button(label=' ' + language.MT_('Reload'),
                              image=get_theme_image('view-refresh-symbolic', Gtk.IconSize.BUTTON))
    reloadbutton.connect('clicked', load_tlp_config, window, True)
    reloadbutton.set_always_show_image(True)
    savebutton = Gtk.Button(label=' ' + language.MT_('Save'),
                            image=get_theme_image('document-save-symbolic', Gtk.IconSize.BUTTON))
    savebutton.connect('clicked', save_tlp_config, window)
    savebutton.set_always_show_image(True)

    settingsbox = Gtk.Box(orientation=Gtk.Orientation.HORIZONTAL, spacing=12)
    settingsbox.pack_start(fileentrylabel, True, True, 0)
    settingsbox.pack_start(reloadbutton, False, False, 0)
    settingsbox.pack_start(savebutton, False, False, 0)

    return settingsbox


def add_menu_actions(window, actiongroup) -> None:
    """Add actions to application menu."""
    actionfilemenu = Gtk.Action("FileMenu", language.MT_("File"), None, None)
    actiongroup.add_action(actionfilemenu)
    actionfilemenusave = Gtk.Action("save", language.MT_('Save'), None, Gtk.STOCK_SAVE)
    actionfilemenusave.connect("activate", save_tlp_config, window)
    actiongroup.add_action(actionfilemenusave)
    actionfilemenuquit = Gtk.Action("quit", language.MT_('Quit'), None, Gtk.STOCK_QUIT)
    actionfilemenuquit.connect("activate", quit_tlp_config, window)
    actiongroup.add_action(actionfilemenuquit)

    actionlanguagemenu = Gtk.Action("LanguageMenu", language.MT_("Language"), None, None)
    actiongroup.add_action(actionlanguagemenu)

    zhlanguagesubmenu = Gtk.Action("zhSubMenu", "zh", None, None)
    actiongroup.add_action(zhlanguagesubmenu)

    actionhelpmenu = Gtk.Action("HelpMenu", language.MT_("Help"), None, None)
    actiongroup.add_action(actionhelpmenu)

    aboutdialogmenu = Gtk.Action("AboutDialog", language.MT_("About"), None, Gtk.STOCK_INFO)
    actiongroup.add_action(aboutdialogmenu)
    aboutdialogmenu.connect('activate', show_about_dialog)

    langdir = Path(settings.langdir)
    for langobject in langdir.iterdir():
        if langobject.is_dir():
            locale = langobject.name

            if locale == settings.userconfig.language:
                actionlang = Gtk.Action(locale, locale, None, Gtk.STOCK_APPLY)
            else:
                actionlang = Gtk.Action(locale, locale, None, None)

            actionlang.connect("activate", switch_language, locale, window)
            actiongroup.add_action(actionlang)


def show_about_dialog(self):
    """Applications about dialog."""
    aboutdialog = Gtk.AboutDialog()
    aboutdialog.set_title("TLP-UI")
    aboutdialog.set_name("name")
    aboutdialog.set_version(get_app_version())
    aboutdialog.set_comments(language.MT_("UI for TLP written in Python/Gtk"))
    aboutdialog.set_website("https://github.com/d4nj1/TLPUI")
    aboutdialog.set_website_label("TLP-UI @ GitHub")
    aboutdialog.set_authors(["Daniel Christophis"])
    aboutdialog.set_translator_credits("Muhammet Emin AKALAN (05akalan57@gmail.com)")
    aboutdialog.set_license_type(Gtk.License.GPL_2_0)
    aboutdialog.set_logo(GdkPixbuf.Pixbuf.new_from_file_at_size(
        f"{settings.icondir}themeable/hicolor/scalable/apps/tlpui.svg", width=128, height=128)
    )
    aboutdialog.connect('response', lambda dialog, fata: dialog.destroy())
    aboutdialog.show_all()


def switch_language(self, lang: str, window: Gtk.Window) -> None:
    """Language switcher."""
    settings.userconfig.language = lang

    # reload language values
    importlib.reload(language)

    load_tlp_config(self, window, False)


def store_option_num(self, option, option_num: int):
    """Store selected functionality option."""
    settings.userconfig.activeoption = option_num


def create_main_box(window: Gtk.Window) -> Gtk.Box:
    """Create TLP configuration items notebook view."""
    notebook = Gtk.Notebook()
    notebook.set_tab_pos(Gtk.PositionType.TOP)

    menubox = create_menu_box(window)
    settingsbox = create_settings_box(window)
    configbox = create_config_box(window)

    contentbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=12)
    contentbox.set_margin_top(18)
    contentbox.set_margin_bottom(18)
    contentbox.set_margin_left(18)
    contentbox.set_margin_right(18)
    contentbox.pack_start(settingsbox, False, False, 0)
    contentbox.pack_start(configbox, True, True, 0)

    configlabel = Gtk.Label(language.MT_('Configuration'))
    configlabel.set_hexpand(True)

    statlabel = Gtk.Label(language.MT_('Statistics'))
    statlabel.set_hexpand(True)

    statbox = create_stat_box()

    notebook.append_page(contentbox, configlabel)
    notebook.append_page(statbox, statlabel)

    mainbox = Gtk.Box(orientation=Gtk.Orientation.VERTICAL, spacing=0)
    mainbox.pack_start(menubox, False, False, 0)
    mainbox.pack_start(notebook, True, True, 0)

    notebook.connect('switch-page', store_option_num)

    activeoption = settings.userconfig.activeoption
    notebook.show_all()
    notebook.set_current_page(activeoption)

    return mainbox



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['tlpui', 'tlpui.ui_config_objects']

package_data = \
{'': ['*'],
 'tlpui': ['configschema/*',
           'defaults/*',
           'icons/*',
           'icons/flags/*',
           'icons/themeable/hicolor/128x128/apps/*',
           'icons/themeable/hicolor/16x16/apps/*',
           'icons/themeable/hicolor/256x256/apps/*',
           'icons/themeable/hicolor/32x32/apps/*',
           'icons/themeable/hicolor/48x48/apps/*',
           'icons/themeable/hicolor/64x64/apps/*',
           'icons/themeable/hicolor/96x96/apps/*',
           'icons/themeable/hicolor/scalable/actions/*',
           'icons/themeable/hicolor/scalable/apps/*',
           'lang/de_DE/LC_MESSAGES/*',
           'lang/en_EN/LC_MESSAGES/*',
           'lang/es_ES/LC_MESSAGES/*',
           'lang/fr_FR/LC_MESSAGES/*',
           'lang/id_ID/LC_MESSAGES/*',
           'lang/pt_BR/LC_MESSAGES/*',
           'lang/ru_RU/LC_MESSAGES/*',
           'lang/tr_TR/LC_MESSAGES/*',
           'lang/zh_CN/LC_MESSAGES/*',
           'lang/zh_TW/LC_MESSAGES/*']}

install_requires = \
['pygobject>=3.46.0,<4.0.0', 'pyyaml>=6.0.1,<7.0.0', 'toml>=0.10.2,<0.11.0']

entry_points = \
{'console_scripts': ['tlpui = tlpui.__main__:main']}

setup_kwargs = {
    'name': 'tlp-ui',
    'version': '1.6.2',
    'description': 'GTK UI for tlp',
    'long_description': '\n<img src="https://raw.githubusercontent.com/d4nj1/TLPUI/master/tlpui/icons/themeable/hicolor/scalable/apps/tlpui.svg" align="left" alt="TLP UI" width="196px">\n\n## TLP UI\n\nThe Python scripts in this project generate a GTK-UI to change [TLP](https://github.com/linrunner/TLP) configuration files easily.\nIt has the aim to protect users from setting bad configuration and to deliver a basic overview of all the valid configuration values.\n\n<img src="https://raw.githubusercontent.com/d4nj1/TLPUI/master/screenshot.png" alt="Screenshot" vspace="20px">\n\n### Install and run instructions :ledger:\n\n<!---* [PyPI](https://github.com/d4nj1/TLPUI/blob/master/docs/INSTALL.md#pypi)-->\n* [Poetry](https://github.com/d4nj1/TLPUI/blob/master/docs/INSTALL.md#poetry)\n* [Python 3](https://github.com/d4nj1/TLPUI/blob/master/docs/INSTALL.md#python-3)\n* [Arch Linux](https://github.com/d4nj1/TLPUI/blob/master/docs/INSTALL.md#arch-linux)\n\n<a href=\'https://flathub.org/apps/details/com.github.d4nj1.tlpui\'><img width=\'240\' alt=\'Download on Flathub\' src=\'https://flathub.org/assets/badges/flathub-badge-en.png\'/></a>\n\n### Current status :sunrise_over_mountains:\n\n* Supports TLP versions 1.3-1.7 - _older TLP versions are supported by [1.6.1](https://github.com/d4nj1/TLPUI/releases/tag/tlpui-1.6.1)_\n* Requires Python 3.8 or greater\n* Configuration can be read and displayed\n* Shows information about configuration changes (defaults/unsaved and drop-in/user config)\n* Changes can be saved with user and sudo permissions (/etc/tlp.conf)\n* tlp-stat can be load in ui (simple and complete)\n\n### To be done :building_construction:\n\n* Weblate translations - [#121](https://github.com/d4nj1/TLPUI/issues/121)\n* Mobile UI - [#111](https://github.com/d4nj1/TLPUI/issues/111)\n* Implement package build pipeline - [#90](https://github.com/d4nj1/TLPUI/issues/90)\n',
    'author': 'Daniel Christophis',
    'author_email': 'code@devmind.org',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://github.com/d4nj1/TLPUI',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8.1,<4.0.0',
}


setup(**setup_kwargs)



"""Provide runtime application settings."""

from os import path, getenv
from pathlib import Path
from . import settingshelper

# application folder settings
workdir = path.dirname(path.abspath(__file__))
langdir = f'{workdir}/lang/'
icondir = f'{workdir}/icons/'

# flatpak related params
IS_FLATPAK = Path("/.flatpak-info").exists()
FOLDER_PREFIX = "/var/run/host" if IS_FLATPAK else ""
TMP_FOLDER = f"{getenv('XDG_RUNTIME_DIR')}/app/{getenv('FLATPAK_ID')}" if IS_FLATPAK else None

# check for required commands to exist
settingshelper.check_binaries_exist(FOLDER_PREFIX)

# user config
userconfig = settingshelper.UserConfig()

# runtime params
tlpversion = settingshelper.get_installed_tlp_version()
tlpbaseversion = tlpversion.replace(".", "_")[0:3]
tlpbaseconfigfile = settingshelper.get_tlp_config_file("")
tlpconfigfile = settingshelper.get_tlp_config_file(FOLDER_PREFIX)
tlpconfig = {}
tlpconfig_original = {}
tlpconfig_defaults = {}
active_scroll = None
